// Copyright 2025 RnD Center "ELVEES", JSC

/*! \file
 *  \brief Тестирование обратного двумерного БПФ для типа fractional16
 */

#include "test.h"

int test_ifft_fract_2d(const int fft_min_size, const int fft_max_size)
{
    int error_count = 0;
    int freq1 = 1;
    int freq2 = 1;
    int16_t *dst = memalign(fft_max_size * 2 * sizeof(int16_t), fft_max_size * 2 * sizeof(int16_t));
    int16_t *src = memalign(fft_max_size * 2 * sizeof(int16_t), fft_max_size * 2 * sizeof(int16_t));
    float *fetalon = memalign(fft_max_size * 2 * sizeof(float), fft_max_size * 2 * sizeof(float));

    char *test_info = malloc(sizeof(char) * 1000000);
    int print_bytes;

    FILE *res_2d;
    if (MEM_TYPE == EXT_EXT)
        res_2d = fopen("test_FRACTIONAL_IFFT_2D_EXT_EXT.csv", "w");
    else
        res_2d = fopen("test_FRACTIONAL_IFFT_2D_EXT_INT.csv", "w");

    fprintf(res_2d, "%s,%s,%s,%s\n", "SIZE", "TIC_CNTR", "INSTR_CNTR", "STATUS");

    print_bytes = sprintf(test_info, "2d fractional IFFT ");
    if (MEM_TYPE == EXT_EXT)
        print_bytes += sprintf(test_info + print_bytes, "EXT_EXT\n\n");
    else
        print_bytes += sprintf(test_info + print_bytes, "EXT_INT\n\n");

    float *v_fl32 = malloc(2 * fft_max_size * sizeof(float));

    int length, height;

    for (int size = fft_min_size; size <= fft_max_size; size <<= 1) {
        length = 1 << (int)log2(size) / 2;
        height = size / length;
        print_bytes += sprintf(test_info + print_bytes, "\nheight = %d, length = %d\n",
                               height, length);
        fprintf(res_2d, "%dx%d,", height, length);

        fft_t fft_obj;
        fft_obj.src = src;
        fft_obj.dst = dst;
        fft_obj.int_mem = (enum CALC_MEM)MEM_TYPE;
        fft_obj.dir = 0;
        fft_obj.dim = (enum Dimension)FFT_DIM;

        memset(src, 0.0, 2 * fft_max_size * sizeof(short));
        src[2 * (freq1 * length + freq2) + 1] = 0.99 * 0.99 * 0x8000;
        if (fft_plan_2d(&fft_obj, height, length, FRACTIONAL) == -1) {
            error_count = 1;
            return error_count;
        };

        fft_execute(&fft_obj);

        if (fft_obj.int_mem && fft_obj.big_size)
            create_spectrum_2d(fetalon, length, height, freq2, freq1, FRACTIONAL);
        else
            create_spectrum_2d(fetalon, height, length, freq1, freq2, FRACTIONAL);

        fract16_to_float32(src, v_fl32, size);

        print_bytes += sprintf(test_info + print_bytes, "\nticks_fft = %d, instr_fft = %d",
                                fft_obj.ticks, fft_obj.instrs);
        fprintf(res_2d, "%d,%d,", fft_obj.ticks, fft_obj.instrs);
        float norm = maximum_norm(v_fl32, fetalon, size);
        print_bytes += sprintf(test_info + print_bytes, ", norm = %f ", norm);
        if (norm < 1e-1) {
            print_bytes += sprintf(test_info + print_bytes, " : OK\n\n");
            fprintf(res_2d, "%s\n", "passed");
        } else {
            ++error_count;
            print_bytes += sprintf(test_info + print_bytes, " : WRONG\n\n");
            fprintf(res_2d, "%s\n", "failed");
        }

        fft_free_sources(&fft_obj);
    }

    printf("%s", test_info);

    free(v_fl32);
    free(src);
    free(dst);
    free(fetalon);
    free(test_info);
    fclose(res_2d);
    return error_count;
}
